/**
 * Copyright 2015 vanita5 <mail@vanita5.de>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Incorporates a modified version of HTML5-Desktop-Notifications
 * (https://github.com/ttsvetko/HTML5-Desktop-Notifications)
 * Original Author: Tsvetan Tsvetkov <tsekach@gmail.com>
 * 
 * 
 * ---
 * name: Mootify 
 * description: HTML Desktop Notifications implementation. Forked from https://github.com/ttsvetko/HTML5-Desktop-Notifications
 * 
 * license: Apache License, Version 2.0
 * 
 * author: vanita5
 * 
 * requires:
 * - MooTools Core
 * 
 * provides: Mootify 
 * 
*/

var Mootify = new Class({
    Implements: [Options, Events],
    constants: {
        PERMISSION_DEFAULT: "default",
        PERMISSION_GRANTED: "granted",
        PERMISSION_DENIED: "denied",
        PERMISSION: [this.PERMISSION_GRANTED, this.PERMISSION_DEFAULT, this.PERMISSION_DENIED]
    },
    options: {
        autoClose: 0,
        isSupported: false,
        title: null,
        body: null,
        icon: null,
        tag: null
    },
    initialize: function(options) {
        this.setOptions(options);
        try {
            this.options.isSupported = (
                window.Notification                                                     //Safari, Chrome
                || window.webkitNotifications                                           //Chrome & ff-html5notifications plugin
                || navigator.mozNotification                                            //Firefox Mobile
                || (window.external && window.external.msIsSiteMode() !== undefined)    //IE9+
            );
        } catch (e) {}
    },
    ieVerification: Math.floor((Math.random() * 10) + 1),
    isString: function (value) {return (value && (value).constructor === String); },
    isFunction: function (value) { return (value && (value).constructor === Function); },
    isObject: function (value) {return (value && (value).constructor === Object); },
    getNotification: function() {
        var notification;
        var options = this.options;

        if (window.Notification) { //Safari 6, Chrome/Chromium 23+
            var icon = this.isString(options.icon) ? options.icon : options.icon.x32;
            notification = new window.Notification(options.title, {
                //Icon of the notification for Chrome
                icon: icon,
                //Message (body) of the notification
                body: options.body || '',
                //Unique identifier for the notification. Prevents duplicate entries of notifications.
                tag: options.tag || ''
            });
        } else if (window.webkitNotifications) { //Firefox with html5Notifications plugin
            notification = window.webkitNotifications.createNotification(options.icon, options.title, options.body);
            notification.show();
        } else if (navigator.mozNotification) { //Firefox Mobile
            notification = navigator.mozNotification.createNotification(options.title, options.body, options.icon);
            notification.show();
        } else if (window.external && window.external.msIsSiteMode()) { //IE9+
            //Clear previous Notifications
            window.external.msSiteModeClearIconOverlay();
            window.external.msSiteModeSetIconOverlay((this.isString(options.icon) ? options.icon : options.icon.x16), options.title);
            window.external.msSiteModeActivate();
            notification = {
                "ieVerification": this.ieVerification + 1
            };
        }
        return notification;
    },
    getWrapper: function(notification) {
        return {
            close: function() {
                if (notification) {
                    if (notification.close) {
                        //http://code.google.com/p/ff-html5notifications/issues/detail?id=58
                        notification.close();
                    }
                    else if (notification.cancel) {
                        notification.cancel();
                    }
                    else if (window.external && window.external.msIsSiteMode()) {
                        if (notification.ieVerification === ieVerification) {
                            window.external.msSiteModeClearIconOverlay();
                        }
                    }
                }
            }
        };
    },
    requestPermission: function(callback) {
        if (!this.options.isSupported) { return; }
        var callbackFunction = this.isFunction(callback) ? callback : function() {};
        if (window.webkitNotifications && window.webkitNotifications.checkPermission) {
            /*
             * Chrome 23 supports Notification.requestPermission
             * but it seems to crash the browser, so we prefer webkitNotifications
             * if available.
             * 
             * html5notifications plugin for Firefox also uses this function.
             */
            window.webkitNotifications.requestPermission(callbackFunction);
        } else if (window.Notification && window.Notification.requestPermission) {
            window.Notification.requestPermission(callbackFunction);
        }
    },
    permissionLevel: function() {
        if (!this.options.isSupported) { return; }
        var permission;
        if (window.Notification && window.Notification.permissionLevel) { //Safari 6
            permission = window.Notification.permissionLevel();
        } else if (window.webkitNotifications && window.webkitNotifications.checkPermission) { //Chrome & FF with html5notifications
            permission = this.constants.PERMISSION[window.webkitNotifications.checkPermission()];
        } else if (window.Notification && window.Notification.permission) { //Firefox 23+
            permission = window.Notification.permission;
        } else if (navigator.mozNotification) { //Firefox Mobile
            permission = this.constants.PERMISSION_GRANTED;
        } else if (window.external && (window.external.msIsSiteMode() !== undefined)) { //IE9+
            permission = window.external.msIsSiteMode() ?
                this.constants.PERMISSION_GRANTED : this.constants.PERMISSION_DEFAULT;
        }
        return permission;
    },
    createNotification: function(options) {
        var notification;
        var notificationWrapper;
        
        this.setOptions(options);
        var options = this.options;

        //Request permission?
        if (this.permissionLevel() == this.constants.PERMISSION_DEFAULT) {
            this.requestPermission(null);
        }

        if (options.isSupported
                && (options && (this.isString(options.icon) || this.isObject(options.icon)))
                && (this.permissionLevel() === this.constants.PERMISSION_GRANTED)) {
            notification = this.getNotification();
        }
        notificationWrapper = this.getWrapper(notification);

        //Auto-close
        if (this.options.autoClose && notification && !notification.ieVerification && notification.addEventListener) {
            notification.addEventListener("show", function () {
                var notification = notificationWrapper;
                window.setTimeout(function () {
                    notification.close();
                }, options.autoClose);
            });
        }
        return notificationWrapper;
    }
});
